@@ -3330,71 +3330,6 @@ _dbus_get_install_root(char *prefix, int len)
     return TRUE;
 }
 
-/** 
-  find config file either from installation or build root according to 
-  the following path layout 
-    install-root/
-      bin/dbus-daemon[d].exe
-      etc/<config-file>.conf *or* etc/dbus-1/<config-file>.conf
-      (the former above is what dbus4win uses, the latter above is
-      what a "normal" Unix-style "make install" uses)
-
-    build-root/
-      bin/dbus-daemon[d].exe
-      bus/<config-file>.conf 
-*/
-dbus_bool_t 
-_dbus_get_config_file_name(DBusString *config_file, char *s)
-{
-  char path[MAX_PATH*2];
-  int path_size = sizeof(path);
-
-  if (!_dbus_get_install_root(path,path_size))
-    return FALSE;
-
-  if(strlen(s) + 4 + strlen(path) > sizeof(path)-2)
-    return FALSE;
-  strcat(path,"etc\\");
-  strcat(path,s);
-  if (_dbus_file_exists(path)) 
-    {
-      // find path from executable 
-      if (!_dbus_string_append (config_file, path))
-        return FALSE;
-    }
-  else 
-    {
-      if (!_dbus_get_install_root(path,path_size))
-        return FALSE;
-      if(strlen(s) + 11 + strlen(path) > sizeof(path)-2)
-        return FALSE;
-      strcat(path,"etc\\dbus-1\\");
-      strcat(path,s);
-  
-      if (_dbus_file_exists(path)) 
-        {
-          if (!_dbus_string_append (config_file, path))
-            return FALSE;
-        }
-      else
-        {
-          if (!_dbus_get_install_root(path,path_size))
-            return FALSE;
-          if(strlen(s) + 4 + strlen(path) > sizeof(path)-2)
-            return FALSE;
-          strcat(path,"bus\\");
-          strcat(path,s);
-          
-          if (_dbus_file_exists(path)) 
-            {
-              if (!_dbus_string_append (config_file, path))
-                return FALSE;
-            }
-        }
-    }
-  return TRUE;
-}    
-
 /* See comment in dbus-sysdeps-unix.c */
 dbus_bool_t
 _dbus_lookup_session_address (dbus_bool_t *supported,
